"""

Test for event objects

Meant to be subclassed

"""
from django.contrib.auth.models import User
from django.test import TestCase

from installations import models as installations_models

from utils import tests as utils_tests
from utils.versioning.models import WorkVersion, Version

class EventTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    ev_model = Event
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)
        self.scope = installations_models.Scope.objects.filter(version__isnull=True)[0]
        self.scenario = self.sc_model(name='scenario', scope=self.scope)
        self.scenario.save(defaultuser)
        self.event = self.ev_model(name='Cause')
        self.event.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.scenario})

    def test_model(self):
        pass

    def test_event_views(self):

        base_url = '/analyses/%s/events/' % self.namespace
        parent_url = '/analyses/%s/scenarios/%s/causetree/' % (self.namespace, self.scenario.v_id)
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class event(utils_tests.TestDefinition):
            description = 'display event'
            url = base_url + '%s/' % self.event.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.event.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.event.v_id, 'users': [1,]},
            ]

        class event_compare(utils_tests.TestDefinition):
            description = 'compare event'
            url = base_url + '%s/compare/0/' % self.event.v_id
            users = [1,2,]

        class event_add(utils_tests.add_TestDefinition):
            description = 'add event'
            url = '/analyses/%s/scenarios/%s/events/add/' % (self.namespace, self.scenario.v_id)
            users = [1,]
            data = {
                "name": "Cause",
                "description": "",
                "management-referer": parent_url
            }
            redirect_url = parent_url
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.ev_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class event_edit(utils_tests.edit_TestDefinition):
            description = 'edit event'
            url = base_url + '%s/edit/' % self.event.v_id
            users = [1,]
            data = {
                "name": "Cause",
                "description": "",
                "management-referer": base_url + '%s/' % self.event.v_id
            }
            redirect_url = base_url + '%s/' % self.event.v_id

        class event_delete(utils_tests.delete_TestDefinition):
            description = 'delete event'
            users = [1,]
            redirect_url = parent_url
            def pretest(self, testcase, allowed):
                newitem = self.test_class.ev_model(name='to_delete')
                newitem.save(defaultuser, treeelementkwargs={'scenario': self.test_class.scenario})
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                if not allowed:
                    self.test_class.ev_model.objects.get(id=self.pk).delete(defaultuser, override=True)

        tests = [
            event(self),
            event_compare(self),
            event_add(self),
            event_edit(self),
            event_delete(self),
        ]

        utils_tests.run_tests(self, users, tests)

