"""

Test for event objects

Meant to be subclassed

"""
from django.contrib.auth.models import User, AnonymousUser
from django.test import TestCase

from installations import models as installations_models

from utils import tests as utils_tests
from utils.versioning.models import WorkVersion, Version

class SearchTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    searchclasses = {}

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)
        self.scope = installations_models.Scope.objects.filter(version__isnull=True)[0]

        self.suggestionlist = self.sl_model(name='SL', description='desc')
        self.suggestionlist.save(defaultuser)

        self.apsuggestionlist = self.apsl_model(name='SL', description='desc')
        self.apsuggestionlist.save(defaultuser)

        # Set up this tree:
        #    Scenario S1 (--scope 1)
        #        -C1|--PL1
        self.treescenario = self.sc_model(name='S1', description='desc', evaluation='eval', scope=self.scope)
        self.treescenario.save(defaultuser)
        self.eventc1 = self.ev_model(name='C1', description='desc')
        self.elc1 = self.eventc1.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.treescenario})
        self.measure = self.me_model(name='M1', description='desc')
        self.measure.save(defaultuser)
        self.protectionlayer = self.pl_model(name='PL1', description='desc')
        self.elpl1 = self.protectionlayer.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc1.pk), treeelementkwargs={'scenario': self.treescenario})
        self.attentionpoint = self.ap_model(name='AP1', description='desc', measure=self.measure)
        self.attentionpoint.save(defaultuser)

    def test_searches(self):
        users = (0,1,2,3,4)

        class base_search(utils_tests.TestDefinition):
            description = ''
            searchitems = []
            users = [1,2,]
            def runtests(self, testcase, allowed, contains, notcontains):
                user = testcase.user or AnonymousUser()
                for item in self.searchitems:
                    res = allowed and item['result'] or 0
                    testcase.assertEquals(len(self.test_class.searchclasses[self.searchclass]().search(item['query'], user, WorkVersion, None)), res)

        class search_suggestionlist(base_search):
            description = 'search a suggestionlist'
            searchclass = 'sl_search'
            searchitems = [
                { "query": 'S', "result": 1},
                { "query": 'desc', "result": 1},
                { "query": 'X', "result": 0},
            ]

        class search_scenario(base_search):
            description = 'search scenario'
            searchclass = 'sc_search'
            searchitems = [
                { "query": 'S', "result": 1},
                { "query": 'desc', "result": 1},
                { "query": 'eval', "result": 1},
                { "query": 'X', "result": 0},
            ]

        class search_event(base_search):
            description = 'search a cause'
            searchclass = 'ev_search'
            searchitems = [
                { "query": 'C', "result": 1},
                { "query": 'desc', "result": 1},
                { "query": 'X', "result": 0},
            ]

        class search_protectionlayer(base_search):
            description = 'search a protectionlayer'
            searchclass = 'pl_search'
            searchitems = [
                { "query": 'P', "result": 1},
                { "query": 'desc', "result": 1},
                { "query": 'X', "result": 0},
            ]

        class search_measure(base_search):
            description = 'search a measure'
            searchclass = 'me_search'
            searchitems = [
                { "query": 'M', "result": 1},
                { "query": 'desc', "result": 1},
                { "query": 'X', "result": 0},
            ]

        class search_apsuggestionlist(base_search):
            description = 'search an attention point suggestionlist'
            searchclass = 'apsl_search'
            searchitems = [
                { "query": 'S', "result": 1},
                { "query": 'desc', "result": 1},
                { "query": 'X', "result": 0},
            ]
        class search_attentionpoint(base_search):
            description = 'search an attention point'
            searchclass = 'ap_search'
            searchitems = [
                { "query": 'A', "result": 1},
                { "query": 'desc', "result": 1},
                { "query": 'X', "result": 0},
            ]

        tests = [
            search_suggestionlist(self),
            search_scenario(self),
            search_event(self),
            search_protectionlayer(self),
            search_measure(self),
            search_apsuggestionlist(self),
            search_attentionpoint(self),
        ]

        utils_tests.run_tests(self, users, tests)
