# utility views

from utils import forms as utils_forms
from utils.versioning import versionedcrud

class TableListView(versionedcrud.TableListView):
    """
    subclass of TableListView, allows using a default table class
    """
    model = None
    default_table_class = None
    default_csv_table_class = None
    default_table_class_attrs = {}
    pdfselectform_class = utils_forms.ListReportForm

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=self.model)

    def get_query_set(self, request):
        return self.model.objects.all().get_versionedobjects(self.version)

    def get_table(self, request, queryset):
        order_by = self.request.GET.get('sort',self.defaultsort)
        if self.table_class:
            table = self.table_class
        else:
            table = utils_forms.create_class_with_meta(self.default_table_class, model=self.model, **self.default_table_class_attrs)
        if table:
            return table(data=queryset, order_by=order_by)
        return None

    def get_csv_table(self, request, queryset):
        order_by = self.request.GET.get('sort',self.defaultsort)
        if self.csv_table_class:
            table = self.csv_table_class
        else:
            table = utils_forms.create_class_with_meta(self.default_csv_table_class, model=self.model)
        if table:
            return table(data=queryset, order_by=order_by)
        return None

class TableListObject(versionedcrud.TableListObject):
    model = None
    default_table_class = None
    default_table_class_attrs = {}
    table_class = None
    defaultsort = None

    def get_table(self, queryset):
        if self.table_class:
            table = self.table_class
        else:
            table = utils_forms.create_class_with_meta(self.default_table_class, model=self.model, **self.default_table_class_attrs)
        if table:
            return table(data=queryset, order_by=self.defaultsort)
        return None
