# dependency views

from analyses.deviations import models as app_models
from analyses.deviations import forms as app_forms
from analyses.deviations import tables as app_tables
from analyses.main import analyses_navigationlist

from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _

from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version

class MixIn(object):
    "Common items for object views"
    model = app_models.Dependency

    def add_to_extra_context(self, extra_context, object):
        pass

class display(MixIn, versionedcrud.DisplayObjectView):
    allow_change = False
    form_class = app_forms.DependencySheet

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.DependencySheet

    def get_default_url(self, object):
        return object.element1.scenario.get_absolute_url('causetree')

class add_to_scenario(MixIn, versionedcrud.AddObjectView):
    form_class = app_forms.DependencyForm
    parent_class = app_models.DeviationScenario
    parent_field = 'scenario'
    parent_tab = 'causetree'

    def setup_parent_from_args(self, object_id=None, *args, **kwargs):
        # scenario is not really a parent here
        # overridden to avoid setting the attribute
        (self.version, self.specialversion) = get_current_version(self.request)
        if self.parent_class and object_id:
            self.parent = versionedcrud.get_versionedobject_or_404(self.parent_class, self.version, v_id = object_id)

    def get_form(self, object, **formargs):
        # override form creation --> pass form the deviationscenario
        return super(add_to_scenario, self).get_form(object, version=self.version, scenario=self.parent)

    def save_form(self, form):
        # scenario is not really a parent here
        # overridden to avoid setting the attribute
        object = form.save(commit=False)
        object = self.save_instance(object)
        form.save_m2m()
        return object


    def get_redirect(self, refform, object):
        return HttpResponseRedirect(self.parent.get_absolute_url('causetree'))

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = app_forms.DependencySheet

views = {
    'display': display(),
    'delete': delete(),
    'add_to_scenario': add_to_scenario(),
    'compare': compare(),
}
