# coding: utf-8

from conversion.connector import get_data

from hazards import models as app_models
from utils.versioning import models as versioning_models
from django.contrib.auth.models import User

from django.db import transaction
import logging
import datetime
from os import path as os_path

# TODO: link and OLE fields

@transaction.commit_on_success
def convert_sd2(sd2_filename = None):

    if sd2_filename:
        defaultuser = User.objects.all()[0]

        logger = logging.getLogger()

        logger.debug('converting sd2 file %s' % sd2_filename)

        # create dictionaries to temporarily store new ids
        substancedict = {}
        reactiondict = {}

        # substances

        sql = """
        SELECT stofid,
        stofnaam, Cas, stoftekst, Rzin, Szin,
        Etikettering
        FROM Stoffen;
        """
        results = get_data(sql, sd2_filename)
        counter = 0
        for row in results:
            if row[1]:
                instances = app_models.Substance.objects.filter(name=row[1], version__isnull=True)
                if not instances:
                    desc = row[3]
                    if desc and row[4]:
                        desc = desc + '\n\nRisk phrases:\n' + row[4]
                    if desc and row[5]:
                        desc = desc + '\n\nSafety phrases:\n' + row[5]
                    if desc and row[6]:
                        desc = desc + '\n\nClassifications:\n' + row[6]
                    instance = app_models.Substance(
                        name = row[1] and row[1][:100],
                        tag = row[2],
                        description = desc,
                        universal = False,
                        )
                    instance.save(defaultuser)
                    substancedict[row[0]] = instance.id
                    counter += 1
        logger.debug('added %s substances' % counter)

        # hazards

        sql = """
        SELECT A.stofid, B.gevarenveld, A.beschrijving
        FROM Stoffen_gevarenvelden A inner join gevarenvelden B on A.gevarenveldid=B.gevarenveldid
        WHERE A.VT;
        """
        results = get_data(sql, sd2_filename)
        counter = 0
        for row in results:
            instid = substancedict.get(row[0])
            if instid:
                try:
                    instance = app_models.Substance.objects.get(pk = instid)
                    if not instance.hazards:
                        instance.hazards = u''
                    if row[1]:
                        instance.hazards += '%s\r\n\r\n' % row[1]
                    if row[2]:
                        instance.hazards += '%s\r\n\r\n\r\n' % row[2]
                    instance.save(defaultuser)
                    counter += 1
                except app_models.Substance.DoesNotExist:
                    pass
        logger.debug('added %s substance hazards' % counter)


        # frequently used substances
        sql = """
        SELECT cmstofid, cmstofnaam
        FROM Cmstoffen;
        """
        results = get_data(sql, sd2_filename)
        counter = 0
        for row in results:
            if row[1]:
                instances = app_models.Substance.objects.filter(name=row[1], version__isnull=True)
                if not instances:
                    instance = app_models.Substance(
                        name = row[1] and row[1][:100],
                        universal = True,
                        )
                    instance.save(defaultuser)
                    substancedict[row[0]] = instance.id
                    counter += 1
        logger.debug('added %s frequently used substances' % counter)

        # interactions

        sql = """
        SELECT A.interactieid,
        A.stofid1, A.stofid2, A.reactieid, A.condities,
        B.reactienaam, B.reactietekst
        FROM interacties A INNER JOIN reacties B ON A.reactieid=B.reactieid;
        """
        results = get_data(sql, sd2_filename)
        counter = 0
        for row in results:
            # find substance 1
            instid1 = substancedict.get(row[1])
            inst1 = None
            if instid1:
                try:
                    inst1 = app_models.Substance.objects.get(pk = instid1, version__isnull=True)
                except app_models.Substance.DoesNotExist:
                    pass
            else:
                if row[1] < 0:
                    sn = get_data("SELECT cmstofnaam FROM cmstoffen WHERE cmstofid=%s" % row[1], sd2_filename)
                    if sn:
                        try:
                            inst1 = app_models.Substance.objects.get(name = sn[0][0], version__isnull=True)
                        except app_models.Substance.DoesNotExist:
                            logger.error('Error: substance %s not found for interaction' % sn)
                else:
                    sn = get_data("SELECT stofnaam FROM stoffen WHERE stofid=%s" % row[1], sd2_filename)
                    if sn:
                        try:
                            inst1 = app_models.Substance.objects.get(name = sn[0][0], version__isnull=True)
                        except app_models.Substance.DoesNotExist:
                            logger.error('Error: substance %s not found for interaction' % sn)

            # find substance 2
            instid2 = substancedict.get(row[2])
            inst2 = None
            if instid2:
                try:
                    inst2 = app_models.Substance.objects.get(pk = instid2)
                except app_models.Substance.DoesNotExist:
                    pass
            else:
                if row[1] < 0:
                    sn = get_data("SELECT cmstofnaam FROM cmstoffen WHERE cmstofid=%s" % row[2], sd2_filename)
                    if sn:
                        try:
                            inst1 = app_models.Substance.objects.get(name = sn[0][0], version__isnull=True)
                        except app_models.Substance.DoesNotExist:
                            logger.error('Error: substance %s not found for interaction' % sn)
                else:
                    sn = get_data("SELECT stofnaam FROM stoffen WHERE stofid=%s" % row[2], sd2_filename)
                    if sn:
                        try:
                            inst1 = app_models.Substance.objects.get(name = sn[0][0], version__isnull=True)
                        except app_models.Substance.DoesNotExist:
                            logger.error('Error: substance %s not found for interaction' % sn)

            if inst1 and inst2:
                # check existing reactions, append number to name if necessary
                instances = app_models.Reaction.objects.filter(name=row[5], version__isnull=True)
                if instances:
                    suffix = u' - %s' % (len(instances)+1)
                    name = u'%s%s' % (row[5][:100-len(suffix)], suffix)
                else:
                    name = row[5] and row[5][:100]

                instance = app_models.Reaction(
                    name = name,
                    description = row[6],
                    substance1 = inst1,
                    substance2 = inst2,
                    conditions = row[4]
                )
                instance.save(defaultuser)
                counter += 1
        logger.debug('added %s interactions' % counter)

        # remaining reactions without interactions

        sql = """
        SELECT reactieid,
        reactienaam, reactietekst
        FROM reacties;
        """
        results = get_data(sql, sd2_filename)
        counter = 0
        for row in results:
            if row[1]:
                instances = app_models.Reaction.objects.filter(name=row[1], version__isnull=True)
                if not instances:
                    instance = app_models.Reaction(
                        name = row[1] and row[1][:100],
                        description = row[2],
                        )
                    instance.save(defaultuser)
                    counter += 1
        logger.debug('added %s reactions' % counter)

        # Now make a version
        version = versioning_models.Version(name='Import from %s' % os_path.basename(sd2_filename))
        version.save(defaultuser)
        logger.debug('End conversion sd2 file')
        return 'Converted sd2 file %s' % sd2_filename

