"""
forms for hazards app
"""
from hazards import models as app_models
from django import forms
from utils import forms as utils_forms
from utils import fields as utils_fields
from utils import lists as utils_lists
from utils import widgets as utils_widgets
from utils import sheets
from utils.versioning.versionedforms import VersionedForm, VersionedSheet
import django.contrib.admin.widgets as adminwidgets
from django.utils.translation import ugettext_lazy as _

class SubstanceFilterForm(utils_lists.FilterForm):
    """Custom filterform for the substances list"""
    name = forms.CharField(max_length=100, required=False, label=_('Substance'),
        widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = 'hazards_substance_name_lookup'
            )
    )
    tag = forms.CharField(max_length=100, required=False, label=_('Tag'))
    class Meta:
        attrs = {
            'name': {'size': 30},
            'tag': {'size': 20},
        }

class SubstanceForm(VersionedForm):
    class Meta:
        model = app_models.Substance
        attrs = {
            'name': {'size': 75},
            'tag': {'size': 40},
            'description': {'rows':5,'cols':75},
            'hazards': {'rows':5,'cols':75},
        }

class SubstanceSheet(VersionedSheet):
    class Meta:
        model = app_models.Substance

class ReactionFilterForm(utils_lists.FilterForm):
    """Custom filterform for the reactions list"""
    name = forms.CharField(max_length=100, required=False, label=_('Reaction'),
        widget = utils_widgets.JQueryAutoCompleteTextInput(
                namedsource = 'hazards_reaction_name_lookup'
            )
        )

class ReactionForm(VersionedForm):
    class Meta:
        model = app_models.Reaction
        attrs = {
            'name': {'size':75},
            'description': {'rows':5,'cols':75},
            'conditions': {'rows':5,'cols':75},
        }
    def __init__(self, version=None, *args, **kwargs):
        super(ReactionForm, self).__init__(*args, **kwargs)
        self.fields['substance1'].queryset = app_models.Substance.objects.get_versionedobjects(version)
        self.fields['substance2'].queryset = app_models.Substance.objects.get_versionedobjects(version)

class ReactionSheet(VersionedSheet):
    class Meta:
        model = app_models.Reaction
        links = ('substance1', 'substance2')
