# installation views

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from utils import crud
from utils.versioning import versionedcrud
from utils.versioning.models import WorkVersion

from installations import models as app_models
from installations import forms as app_forms
from installations import tables as app_tables
from installations.views.main import installations_navigationlist

class MixIn(object):
    "Common items for object views"
    model = app_models.Section_Reaction

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['breadcrumbs'] = [{'url': object.section.get_absolute_url('reactions'),'label': u'%s' % object.section}]

class display(MixIn, versionedcrud.DisplayObjectView):
    form_class = app_forms.Section_ReactionSheet
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.Section_ReactionForm

    def get_form(self, object):
        return super(edit, self).get_form(object, version=WorkVersion)

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.Section_ReactionSheet
    def get_default_url(self, object):
        return object.section.get_absolute_url('reactions')

class add_to_section(MixIn, versionedcrud.AddObjectView):
    form_class = app_forms.Section_ReactionForm
    parent_class = app_models.Section
    parent_field = 'section'
    parent_tab = 'reactions'

    def get_form(self, object):
        return super(add_to_section, self).get_form(object, version=WorkVersion)

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = app_forms.Section_ReactionSheet

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)
