# group views

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from users import forms as app_forms
from users import models as app_models
from users import tables as app_tables
from users.views.user import users_navigationlist
from utils import crud
from utils.autocomplete import autocomplete

class MixIn(object):
    "Common items for object views"
    model = app_models.Group

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  users_navigationlist()
    tabs = [
        {'name': 'group',
            'label': _('Group')},
        {'name': 'users',
            'label': _('Members')},
    ]

class display_group(MixIn, crud.DisplayObjectView):
    form_class = app_forms.GroupSheet

class display_users(MixIn, crud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    form_class = None

    def add_to_extra_context(self, extra_context, object):
        crud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)
        queryset = app_models.User.objects.filter(groups=object)
        table = app_tables.UsersTable(queryset)
        extra_context['table'] = table

class display(MixIn, crud.TabContainer):
    tabviews = {
        'group': display_group(),
        'users': display_users(),
    }
    default = 'group'

class edit(MixIn, crud.EditObjectView):
    form_class = app_forms.GroupForm
    tabs = []

class delete(MixIn, crud.DeleteObjectView):
    form_class = app_forms.GroupSheet
    tabs = []

    def get_default_url(self, object):
        return reverse('users_groups')

class add(MixIn, crud.AddObjectView):
    form_class = app_forms.GroupForm
    tabs = []

class GroupFilter(crud.Filter):
    form = app_forms.GroupFilterForm
    filters = [
                {'filteritem': 'name', 'parameter': 'name__icontains', 'type': 'normal'},
            ]

class groups(crud.TableListView):
    "List of groups"

    table_class = app_tables.GroupsTable
    defaultsort = 'name'
    queryset = app_models.Group.objects.all()
    paginate_by = 100
    title = _('Groups')
    filter = GroupFilter()
    redirect_on_unique = True

    def add_to_extra_context(self, extra_context):
        super(groups, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  users_navigationlist('groups')
        if app_models.Group.may_add(self.request.user):
            extra_context['tablebuttons'] = [{
                    'url': reverse('users_group_add'),
                    'label': _('Add'),
                }]
