from django.template import Library

from django.utils.translation import ugettext_lazy as _

from utils.attachments.models import Attachment
from utils.attachments.views import display_url_for_obj

register = Library()

@register.inclusion_tag('attachments/display_link.html', takes_context=True)
def attachments_link(context, obj):
    """
    Renders a link to the objects attachments list.

    """
    att_count = Attachment.objects.attachments_for_object(obj).count()
    if att_count == 1:
        name = Attachment._meta.verbose_name
    else:
        name = Attachment._meta.verbose_name_plural

    return {
        'MEDIA_URL': context['MEDIA_URL'],
        'display_label': _('%(count)s %(name)s') % {'count': att_count, 'name': name},
        'display_url': display_url_for_obj(obj),
    }

