# news views

from django.core.urlresolvers import reverse
from django.db.models.loading import get_model
from django.utils.translation import ugettext_lazy as _

from utils import crud
import utils.news.models as app_models
import utils.news.forms as app_forms
import utils.news.tables as app_tables
from utils import permissions
from utils import crud

class display(crud.DisplayObjectView):
    model = app_models.Entry
    template = 'news/entry.html'

class edit(crud.EditObjectView):
    model = app_models.Entry
    form_class = app_forms.EntryForm

    def save_instance(self, object):
        """
        Make sure the user is passed into the object save function
        """
        object.save(self.request.user)
        return object

class delete(crud.DeleteObjectView):
    model = app_models.Entry
    form_class = app_forms.EntrySheet

    def get_default_url(self, object):
        return reverse('news_entries')

class add(crud.AddObjectView):
    """
    Add a news entry
    """
    model = app_models.Entry
    form_class = app_forms.EntryForm

    def save_instance(self, object):
        """
        Make sure the user is passed into the object save function
        """
        object.save(self.request.user)
        return object

class entries(crud.TableListView):
    "List of news entries"

    table_class = app_tables.EntriesTable
    defaultsort = '-date_created'
    queryset = app_models.Entry.objects.all()
    paginate_by = 20
    title = _('News')

    def add_to_extra_context(self, extra_context):
        super(entries, self).add_to_extra_context(extra_context)
        if app_models.Entry.may_add(self.request.user):
            extra_context['tablebuttons'] = [{
                    'url': reverse('news_entry_add'),
                    'label': _('Add'),
                }]
