"""
string manipulation helpers
"""

from django.utils import translation

def ugettext_tolanguage(string, language):
    """
    >>> ugettext_tolanguage('Cause', 'en')
    u'Cause'

    >>> ugettext_tolanguage('Cause', 'nl')
    u'Oorzaak'

    >>> ugettext_tolanguage('Cause', 'bogus')
    u'Cause'
    """
    cur_language = translation.get_language()
    text = string
    try:
        translation.activate(language)
        text = translation.ugettext(string)
    finally:
        translation.activate(cur_language)
    return text


def add_suffix_to_string(string, suffix, max_length):
    """
    >>> add_suffix_to_string(u'xxx', u'yy', 10)
    u'xxx yy'

    >>> add_suffix_to_string(u'xxx', u'yy', 4)
    u'x yy'

    >>> add_suffix_to_string(u'xxx', u'', 10)
    u'xxx'

    >>> add_suffix_to_string(u'xxx', u'', 2)
    u'xx'
    """
    if suffix:
        ml = max_length - len(suffix) - 1
        return u'%s %s' % (string[:ml], suffix)
    else:
        return string[:max_length]

def trim_string(string, max_length):
    """
    >>> trim_string(u'xxx', 10)
    u'xxx'

    >>> trim_string(u'xxx', 2)
    u'xx...'

    """
    if len(string) > max_length:
        return u'%s...' % string[:max_length]
    return string

def count_ident(str1, str2):
    """
    return the number of identical characters in two strings
    >>> count_ident(u'a', u'b')
    0

    >>> count_ident(u'aa', u'ab')
    1

    >>> count_ident(u'aa', u'aa')
    2

    >>> count_ident(u'aa', u'aab')
    2

    >>> count_ident(u'abab', u'a')
    1

    """
    l = min(len(str1), len(str2))
    for i in range(l):
        if str1[i:i+1] != str2[i:i+1]:
            return i
    return l